; Pong Solo v1.2 for TSE
; By Robin Kay
;

; You may not distribute a modified version of this program. However, you
; may take code from or base your program on this code providing that the
; resulting program is at least 50% of your own work.

.nolist
#include "tse.inc"
.list
.org    userMem-3
.db     $bb,$6d
 ret

.db "TSE",1              ; header
prgtitle:
.db " Pong Solo", 0      ; title
.dw 58                   ; stack required

paddle_y .equ 63-2

start:

 ; Dispaly main menu
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl
 ld hl, prgtitle + 1
 call putStr
 ld hl, author
 call putStr
 bcall(_newline)
 ld hl, txt1
 call putStr
 ld hl, txt2
 call putStr
 bcall(_newline)
 bcall(_newline)
 ld hl, hstxt
 bcall(_puts)
 ld hl, (highscore)
 bcall(_disphl)

 ; Poll for keypad on menu
loop:
 ei
 halt ; Save power
 bcall(_getcsc)

 cp sk1
 jr nz, cont0
 ld h, 10
 jr maingame
cont0:

 cp sk2
 jr nz, cont1
 ld h, 5
 jr maingame
cont1:

 cp sk3
 jr nz, cont2
 ld h, 2
 jr maingame
cont2:

 ; Process other keys
 cp skclear
 jr z, clrhs

 ; Yield
 cp skStat
 jr nz, dontyield
 call _tseforceyield
 jr start
dontyield:
 jr loop

; Clear highscore
clrhs:
 ld hl, 0
 ld (highscore), hl
 jr start 

; Main game routine
maingame:

 ; Save delay
 ld l, 0
 ld (delay), hl

 ; Reset score
 ld hl, 0
 ld (score), hl

 ; Init variables
 ld b, 20
 call ionRandom
 inc a
 ld (bally), a ; Intialize the ball y (1-20) -- top to bottom
 ld b, 80
 call ionRandom
 inc a
 ld (ballx), a ; Intializes the ball x (1-80) -- left to right
 and %11111110
 ld (paddlex), a ; Initialize the user paddle x (0-79) -- left to right
 ld a, -1
 ld (ballvx), a ; Iitialize the ball velocity x
 ld (ballvy), a ; Initialize the ball velocity y

gameloop:
 ; Clear the screen
 bcall(_GrBufClr)

 ; Update X position of ball
 ld a, (ballvx)
 ld b, a
 ld a, (ballx)
 add a, b
 ld (ballx), a

 ; Check for collisions with wall
 cp 2
 call m, bouncex
 cp 96-8
 call p, bouncex

 ; Update Y position of ball
 ld a, (ballvy)
 ld b, a
 ld a, (bally)
 add a, b
 ld (bally), a

 ; Check for collisions with top wall and paddle
 cp 2
 call m, bouncey_top
 cp paddle_y - 4
 call p, bouncey_paddle

 ; Check for Group 1 keypresses
 ld a, kReset
 out (1), a
 ld a, Group1
 out (1), a                
 in a,(1)

 ; Move the user's paddle left
 cp kLeft
 jr nz, noleft
 ld a, (paddlex)
 sub 2
 cp 0
 jr z, noleft
 ld (paddlex),a
noleft:

 ; Move the user's paddle right
 cp kRight
 jr nz, noright
 ld a,(paddlex)
 add a, 2
 cp 96-16
 jr z, noright
 ld (paddlex), a
noright:

 ; Check for Group 2 keypresses
 ld a, kReset
 out (1), a
 ld a, Group2
 out (1), a                
 in a,(1)
 cp kClear
 jp z, start
	
 ; Draw the ball
 ld ix, ball
 ld a, (bally)
 ld l, a
 ld a, (ballx)
 ld b, 4
 call ionPutSprite

 ; Draw the user's paddle
 ld ix, paddle
 ld l, paddle_y
 ld a, (paddlex)
 ld b, 2
 ld c, 2
 call ionLargeSprite

 ; Draw the buffer to the screen
 call ionFastCopy

 ; Delay the game
 ld bc, (delay)
 ld hl, 8000h
 ld d, h
 ld e, l
 ldir ; Nop!

 ; Yield
 ld a,kreset
 out (1),a
 ld a,group4
 out (1),a
 in a,(1)
 cp kStat
 call z, _tseforceyield
 jp gameloop

 ; Bounce the ball off the side of the screen
bouncex:
 ld a, (ballvx)
 neg
 ld (ballvx), a
 ret

bouncey_top:
 ; Bounce the ball off the top of the screen
 ld a, 1
 ld (ballvy),a
 ret

bouncey_paddle:
 ; Bounce the ball off the user paddle
 ld a, -1
 ld (ballvy), a

 ; Check if ball is off left side of paddle
 ld a,(ballx) 
 add a, 8 
 ld b, a
 ld a, (paddlex)
 cp b
 jr nc, done

 ; Check if ball is off right side of paddle
 ld a, (ballx) 
 ld b, a   
 ld a, (paddlex) 
 add a, 16
 cp b     
 jr c, done

 ; Update score
 ld hl, (score)
 inc hl
 ld (score), hl
 ret

; Hiscore screen
done:
 pop hl ; Remove return address from stack
doneLoop:
 bcall(_clrscrnfull)
 ld hl, 0
 ld (currow), hl
 ld hl, scrtxt
 bcall(_puts)
 ld hl, (score)
 bcall(_disphl)
 bcall(_newline)
 ld hl, (score)
 ld de, (highscore)
 inc de
 bcall(_cphlde)
 jr c, nohighscore

 ld (highscore), hl
 ld hl, newhs
 bcall(_puts)

nohighscore:
 ei
 halt ; Save power
 bcall(_getcsc)
 cp skMode
 jp z, start
 cp skStat
 jr nz, nohighscore
 call z, _tseforceyield
 jr doneLoop

; Display string and then advance a line onwards
putStr:
 bcall(_puts)
 bcall(_newline)
 ret

highscore:
.dw 0

ball:
.db %00111100
.db %01111110
.db %01111110
.db %00111100

paddle:
.db %11011111,%11111011
.db %11011111,%11111011

author:
.db "By Robin Kay", 0
txt1:
.db "1-3: Start Game",0
txt2:
.db "Clr: Reset HS",0
hstxt:
.db "Hi-"
scrtxt
.db "Score: ",0
newhs:
.db "New HighScore!!",0

extdata:
paddlex .equ extdata   ; X of paddle
ballx   .equ paddlex+1 ; X of ball
bally   .equ ballx+1   ; Y of ball
ballvx  .equ bally+1   ; X velocity of ball
ballvy  .equ ballvx+1  ; Y velocity of ball
score   .equ ballvy+1  ; Score
delay   .equ score+2   ; Delay in main game loop

.end
